function  [sys, x0, str, ts]  = rlsests(t,x,u,flag,nstates,lambda,dt)
%RLSESTS S-function to perform system identification.
%
%   This M-file is designed to be used in a Simulink S-function block.
%   It performs parameter estimation using the Recursive Least Squares
%   Parameter Estimation Algorithm with Exponential Data Weighting
%   
%   The input arguments are
%
%     nstates:        the number of states in the states vector
%     lambda:         the exponential data weighting factor
%     dt:             how often to sample points (secs)
%
%   The RLS estimator is defined by the following equations:
%
%   	                        1      P(k-2) * phi(k-1) * [y(k) - phi(k-1)'theta(k-1)]
%     theta[k] = theta[k-1] + ------ * -------------------------------------------------
%   	                      lambda         lambda + phi(k-1)' * P(k-2) * phi(k-1)
%
%   	         1       P(k-2) * phi(k-1) * phi(k-1)' * P(k-2)
%     P(k-1) = ------ * ----------------------------------------
%   	       lambda    lambda + phi(k-1)' * P(k-2) * phi(k-1)
%
%   where:
%
%     theta:		the parameter estimates
%     phi:		the state vector
%     P:		the covariance matrix
%     lambda:		the exponential data weighting factor
%
%   See also SFUNTMPL., "Adaptive Filtering, Prediction, and Control",
%   G. C. Goodwin & K. S. Sin.

%   Copyright (c) 1990-1998 by The MathWorks, Inc. All Rights Reserved.
%   $Revision: 1.15 $
%   Rick Spada 6-17-92.


if abs(flag) == 2	% flag = 2 --> real time hit
  
  % sample hit, return the next discrete states, which are the
  % next parameter estimates

  theta = x(1:nstates);					% parameter estimates
  P = zeros(nstates,nstates);				% get covariance matrix
  P(:) = x(nstates+1:nstates+nstates*nstates);
  yk = u(nstates + 1);					% system output
  phi = u(1:nstates);					% state vector
  est_err = yk - phi' * theta;				% estimation error
  den = lambda + phi' * P * phi;			% lambda + phi' * P * phi
  theta_new = theta + P * phi * (est_err / den);	% nfunction [sys, x0, str, ts] = rlspps(t,x,u,flag,name,order,char_poly,dt)
%RLSPPS S-function which performs pole placement.
%   This M-file is designed to be used in a Simulink S-function block.
%   It computes the gains for a filter placed in the feed forward path
%   of a control system such that the closed loop has the desired
%   characteristic polynomial.  The gains are returned as a single
%   vector with P as the first half, and L as the second half.
%   
%   The input arguments are
%   sys_name:   the name of the block diagram system
%   blk_name:   the name of the discrete controller in the block diagram
%   order:      the order of the model
%   ic:         the initial condition for the model parameters
%   char_poly:  the desired characteristic polynomial
%   dt:         how often to sample points (secs)
%
%   See also SFUNTMPL.

%   Copyright (c) 1990-1998 by The MathWorks, Inc. All Rights Reserved.
%   $Revision: 1.15 $
%   Rick Spada 6-17-92

if abs(flag) == 2
